@@ -1,6 +1,6 @@
 /****************************************************************/
 /* Debug unit                                                   */
-/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-04               */
+/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-05               */
 /****************************************************************/
 
 /*
@@ -403,7 +403,8 @@ void SU_DBG_PrintDebug(const SU_u64 Type,char *Txt, ...)
         atom = GlobalAddAtom(Str2);
         if(atom != 0)
         {
-          PostMessage(SU_DBG_OUT_CONSOLE_Hwnd,SU_DBG_OUT_CONSOLE_Msg,atom,(LPARAM)Type);
+          /*PostMessage(SU_DBG_OUT_CONSOLE_Hwnd,SU_DBG_OUT_CONSOLE_Msg,atom,(LPARAM)Type);*/
+          SendMessage(SU_DBG_OUT_CONSOLE_Hwnd,SU_DBG_OUT_CONSOLE_Msg,atom,(LPARAM)Type);
         }
       }
     }
