@@ -1,6 +1,6 @@
 /****************************************************************/
 /* Win32 registry functions                                     */
-/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-04               */
+/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-05               */
 /****************************************************************/
 
 /*
@@ -165,56 +165,222 @@ HKEY SU_RB_CreateKeys(const char Key[])
   return H;
 }
 
-void SU_RB_GetStrValue(const char Key[],char *buf,int buf_len,const char Default[])
+bool SU_RB_GetStrValue(const char Key[],char *buf,int buf_len,const char Default[])
 {
   HKEY Handle;
   char *p;
   DWORD VType;
   DWORD Size;
-
-  SU_strcpy(buf,Default,buf_len);
-  Handle = SU_RB_OpenKeys(Key,0);
-  if(Handle == 0)
-    return;
-  p = strrchr(Key,'\\');
-  if(p == NULL)
-  {
-    RegCloseKey(Handle);
-    SU_RB_LastError = SU_RB_ERR_INVALID_KEY;
-    return;
-  }
-  p++;
-  Size = buf_len;
-  RegQueryValueEx(Handle,p,NULL,&VType,buf,&Size);
-  RegCloseKey(Handle);
-}
-
-int SU_RB_GetIntValue(const char Key[],int Default)
-{
-  HKEY Handle;
-  char *p;
-  int Value;
-  DWORD VType;
-  DWORD Size;
   long R;
 
+  SU_strcpy(buf,Default,buf_len);
   Handle = SU_RB_OpenKeys(Key,0);
   if(Handle == 0)
-    return Default;
+    return true; /* Returns true even if key not there, with the Default value */
   p = strrchr(Key,'\\');
   if(p == NULL)
   {
     RegCloseKey(Handle);
     SU_RB_LastError = SU_RB_ERR_INVALID_KEY;
-    return Default;
+    return false;
   }
   p++;
-  Size = sizeof(Value);
-  R = RegQueryValueEx(Handle,p,NULL,&VType,(BYTE *)&Value,&Size);
+  Size = buf_len;
+  if(RegQueryValueEx(Handle,p,NULL,&VType,NULL,NULL) == ERROR_SUCCESS) /* Only check for Type if no error */
+  {
+    if(VType != REG_SZ)
+    {
+      RegCloseKey(Handle);
+      SU_RB_LastError = SU_RB_ERR_WRONG_TYPE;
+      return false;
+    }
+  }
+  R = RegQueryValueEx(Handle,p,NULL,NULL,buf,&Size);
   RegCloseKey(Handle);
   if(R != ERROR_SUCCESS)
-    return Default;
-  return Value;
+  {
+    if(R == ERROR_MORE_DATA)
+    {
+      SU_RB_LastError = SU_RB_ERR_MORE_DATA;
+      return false;
+    }
+  }
+  return true; /* Returns true even if key not there, with the Default value */
+}
+
+bool SU_RB_GetStrLength(const char Key[],int *length)
+{
+  HKEY Handle;
+  char *p;
+  DWORD VType;
+  long R;
+
+  Handle = SU_RB_OpenKeys(Key,0);
+  if(Handle == 0)
+  {
+    SU_RB_LastError = SU_RB_ERR_NO_SUCH_KEY;
+    return false;
+  }
+  p = strrchr(Key,'\\');
+  if(p == NULL)
+  {
+    RegCloseKey(Handle);
+    SU_RB_LastError = SU_RB_ERR_INVALID_KEY;
+    return false;
+  }
+  p++;
+  if(RegQueryValueEx(Handle,p,NULL,&VType,NULL,NULL) == ERROR_SUCCESS) /* Only check for Type if no error */
+  {
+    if(VType != REG_SZ)
+    {
+      RegCloseKey(Handle);
+      SU_RB_LastError = SU_RB_ERR_WRONG_TYPE;
+      return false;
+    }
+  }
+  R = RegQueryValueEx(Handle,p,NULL,NULL,NULL,length);
+  RegCloseKey(Handle);
+  if(R != ERROR_SUCCESS)
+  {
+    if(R == ERROR_MORE_DATA)
+    {
+      SU_RB_LastError = SU_RB_ERR_MORE_DATA;
+      return false;
+    }
+  }
+  return true;
+}
+
+bool SU_RB_GetIntValue(const char Key[],int *Value,int Default)
+{
+  HKEY Handle;
+  char *p;
+  int Val;
+  DWORD VType;
+  DWORD Size;
+  long R;
+
+  *Value = Default;
+  Handle = SU_RB_OpenKeys(Key,0);
+  if(Handle == 0)
+    return true; /* Returns true even if key not there, with the Default value */
+  p = strrchr(Key,'\\');
+  if(p == NULL)
+  {
+    RegCloseKey(Handle);
+    SU_RB_LastError = SU_RB_ERR_INVALID_KEY;
+    return false;
+  }
+  p++;
+  Size = sizeof(Val);
+  if(RegQueryValueEx(Handle,p,NULL,&VType,NULL,NULL) == ERROR_SUCCESS) /* Only check for Type if no error */
+  {
+    if(VType != REG_DWORD)
+    {
+      RegCloseKey(Handle);
+      SU_RB_LastError = SU_RB_ERR_WRONG_TYPE;
+      return false;
+    }
+  }
+  R = RegQueryValueEx(Handle,p,NULL,NULL,(BYTE *)&Val,&Size);
+  if(R == ERROR_SUCCESS)
+    *Value = Val;
+  RegCloseKey(Handle);
+  return true; /* Returns true even if key not there, with the Default value */
+}
+
+bool SU_RB_GetBinValue(const char Key[],char *buf,int buf_len,char *Default,int def_len)
+{
+  HKEY Handle;
+  char *p;
+  DWORD VType;
+  DWORD Size;
+  long R;
+
+  if(Default != NULL)
+  {
+    memcpy(buf,Default,buf_len);
+    if(buf_len < def_len)
+    {
+      SU_RB_LastError = SU_RB_ERR_MORE_DATA;
+      return false;
+    }
+  }
+  Handle = SU_RB_OpenKeys(Key,0);
+  if(Handle == 0)
+    return true; /* Returns true even if key not there, with the Default value */
+  p = strrchr(Key,'\\');
+  if(p == NULL)
+  {
+    RegCloseKey(Handle);
+    SU_RB_LastError = SU_RB_ERR_INVALID_KEY;
+    return false;
+  }
+  p++;
+  Size = buf_len;
+  if(RegQueryValueEx(Handle,p,NULL,&VType,NULL,NULL) == ERROR_SUCCESS) /* Only check for Type if no error */
+  {
+    if(VType != REG_BINARY)
+    {
+      RegCloseKey(Handle);
+      SU_RB_LastError = SU_RB_ERR_WRONG_TYPE;
+      return false;
+    }
+  }
+  R = RegQueryValueEx(Handle,p,NULL,NULL,buf,&Size);
+  RegCloseKey(Handle);
+  if(R != ERROR_SUCCESS)
+  {
+    if(R == ERROR_MORE_DATA)
+    {
+      SU_RB_LastError = SU_RB_ERR_MORE_DATA;
+      return false;
+    }
+  }
+  return true; /* Returns true even if key not there, with the Default value */
+}
+
+bool SU_RB_GetBinLength(const char Key[],int *length)
+{
+  HKEY Handle;
+  char *p;
+  DWORD VType;
+  long R;
+
+  Handle = SU_RB_OpenKeys(Key,0);
+  if(Handle == 0)
+  {
+    SU_RB_LastError = SU_RB_ERR_NO_SUCH_KEY;
+    return false;
+  }
+  p = strrchr(Key,'\\');
+  if(p == NULL)
+  {
+    RegCloseKey(Handle);
+    SU_RB_LastError = SU_RB_ERR_INVALID_KEY;
+    return false;
+  }
+  p++;
+  if(RegQueryValueEx(Handle,p,NULL,&VType,NULL,NULL) == ERROR_SUCCESS) /* Only check for Type if no error */
+  {
+    if(VType != REG_BINARY)
+    {
+      RegCloseKey(Handle);
+      SU_RB_LastError = SU_RB_ERR_WRONG_TYPE;
+      return false;
+    }
+  }
+  R = RegQueryValueEx(Handle,p,NULL,NULL,NULL,length);
+  RegCloseKey(Handle);
+  if(R != ERROR_SUCCESS)
+  {
+    if(R == ERROR_MORE_DATA)
+    {
+      SU_RB_LastError = SU_RB_ERR_MORE_DATA;
+      return false;
+    }
+  }
+  return true;
 }
 
 bool SU_RB_SetStrValue(const char Key[],const char Value[])
@@ -279,6 +445,37 @@ bool SU_RB_SetIntValue(const char Key[],int Value)
   }
 }
 
+bool SU_RB_SetBinValue(const char Key[],const char *Value,int val_len)
+{
+  HKEY Handle;
+  char *p;
+  long R;
+
+  Handle = SU_RB_CreateKeys(Key);
+  if(Handle == 0)
+    return false;
+  p = strrchr(Key,'\\');
+  if(p == NULL)
+  {
+    RegCloseKey(Handle);
+    SU_RB_LastError = SU_RB_ERR_INVALID_KEY;
+    return false;
+  }
+  p++;
+  R = RegSetValueEx(Handle,p,0,REG_BINARY,Value,val_len);
+  RegCloseKey(Handle);
+  if(R == ERROR_SUCCESS)
+  {
+    SU_RB_LastError = SU_RB_ERR_SUCCESS;
+    return true;
+  }
+  else
+  {
+    SU_RB_LastError = SU_RB_ERR_ACCESS_DENIED;
+    return false;
+  }
+}
+
 bool SU_RB_DelKey(const char Key[])
 {
   HKEY Handle;
@@ -357,7 +554,7 @@ bool SU_RB_EnumStrValue(HKEY Key,int Idx,char *Name,int name_len,char *Value,int
 {
   int len = name_len;
   int len2 = value_len;
-  LONG ret = RegEnumValue(Key,Idx,Name,&len,NULL,NULL,Value,&len2);
+  LONG ret = RegEnumValue(Key,Idx,Name,(unsigned long *)&len,NULL,NULL,Value,(unsigned long *)&len2);
 
   return(ret == ERROR_SUCCESS);
 }
@@ -366,7 +563,7 @@ bool SU_RB_EnumIntValue(HKEY Key,int Idx,char *Name,int name_len,int *Value) /*
 {
   int len = name_len;
   int len2 = sizeof(Value);
-  LONG ret = RegEnumValue(Key,Idx,Name,&len,NULL,NULL,(unsigned char *)Value,&len2);
+  LONG ret = RegEnumValue(Key,Idx,Name,(unsigned long *)&len,NULL,NULL,(unsigned char *)Value,(unsigned long *)&len2);
 
   return(ret == ERROR_SUCCESS);
 }
@@ -387,6 +584,7 @@ bool SU_RB_CloseRegistry(void)
 
 #define SU_RB_TYPE_INT 1
 #define SU_RB_TYPE_STR 2
+#define SU_RB_TYPE_BUF 3
 
 /* ********* */
 #include <sys/file.h>
@@ -616,7 +814,7 @@ HKEY SU_RB_CreateKeys(const char Key[]) /* Created HKEY or 0 on error */
 
 bool SU_RB_EnumKey(HKEY Key,int Idx,char *Name,int name_len) /* True on Success. False when no more values available */
 {
-  fprintf(stderr,"SU_RB_EnumKey : TO DO !! Abord\n");
+  fprintf(stderr,"SU_RB_EnumKey : TO DO !! Abort\n");
   abort();
 }
 
@@ -984,6 +1182,11 @@ SU_RB_PValue _SU_RB_ReadStrValue(SU_RB_PNode Node,const char Key[],char *buf,int
     Val = (SU_RB_PValue)Ptr->Data;
     if(SU_strcasecmp(Key,Val->Name))
     {
+      if(Val->Type != SU_RB_TYPE_STR)
+      {
+        SU_RB_LastError = SU_RB_ERR_WRONG_TYPE;
+        return NULL;
+      }
       if(buf != NULL)
         SU_strcpy(buf,(char *)Val->Value,buf_len);
       return Val;
@@ -1007,6 +1210,11 @@ SU_RB_PValue _SU_RB_ReadIntValue(SU_RB_PNode Node,const char Key[],int *val)
     Val = (SU_RB_PValue)Ptr->Data;
     if(SU_strcasecmp(Key,Val->Name))
     {
+      if(Val->Type != SU_RB_TYPE_INT)
+      {
+        SU_RB_LastError = SU_RB_ERR_WRONG_TYPE;
+        return NULL;
+      }
       if(val != NULL)
         *val = (int)Val->Value;
       return Val;
@@ -1061,7 +1269,7 @@ bool _SU_RB_SetIntValue(SU_RB_PNode Node,const char Key[],const int Value)
   return true;
 }
 
-void SU_RB_GetStrValue(const char Key[],char *buf,int buf_len,const char Default[])
+bool SU_RB_GetStrValue(const char Key[],char *buf,int buf_len,const char Default[])
 {
   SU_RB_PNode Handle;
   char *p;
@@ -1069,26 +1277,39 @@ void SU_RB_GetStrValue(const char Key[],char *buf,int buf_len,const char Default
   SU_strcpy(buf,Default,buf_len);
   Handle = SU_RB_OpenKeys(Key,0);
   if(Handle == NULL)
-    return;
+    return true; /* Returns true even if key not there, with the Default value */
   p = strrchr(Key,'\\');
   if(p == NULL)
   {
     SU_RB_LastError = SU_RB_ERR_INVALID_KEY;
-    return;
+    return false;
   }
   p++;
-  _SU_RB_ReadStrValue(Handle,p,buf,buf_len);
+  SU_RB_LastError = SU_RB_ERR_SUCCESS;
+  if(_SU_RB_ReadStrValue(Handle,p,buf,buf_len) == NULL)
+  {
+    if(SU_RB_LastError == SU_RB_ERR_WRONG_TYPE)
+      return false;
+  }
+  return true; /* Returns true even if key not there, with the Default value */
 }
 
-int SU_RB_GetIntValue(const char Key[],int Default)
+bool SU_RB_GetStrLength(const char Key[],int *length)
+{
+  fprintf(stderr,"SU_RB_GetStrLength : TO DO !! Abort\n");
+  abort();
+}
+
+bool SU_RB_GetIntValue(const char Key[],int *Value,int Default)
 {
   SU_RB_PNode Handle;
   char *p;
   int val;
 
+  *Value = Default;
   Handle = SU_RB_OpenKeys(Key,0);
   if(Handle == NULL)
-    return Default;
+    return true; /* Returns true even if key not there, with the Default value */
   p = strrchr(Key,'\\');
   if(p == NULL)
   {
@@ -1096,10 +1317,27 @@ int SU_RB_GetIntValue(const char Key[],int Default)
     return Default;
   }
   p++;
-  if(_SU_RB_ReadIntValue(Handle,p,&val))
-    return val;
+  SU_RB_LastError = SU_RB_ERR_SUCCESS;
+  if(_SU_RB_ReadIntValue(Handle,p,&val) == NULL)
+  {
+    if(SU_RB_LastError == SU_RB_ERR_WRONG_TYPE)
+      return false;
+  }
   else
-    return Default;
+    *Value = val;
+  return true; /* Returns true even if key not there, with the Default value */
+}
+
+bool SU_RB_GetBinValue(const char Key[],char *buf,int buf_len,char *Default,int def_len)
+{
+  fprintf(stderr,"SU_RB_GetBinValue : TO DO !! Abort\n");
+  abort();
+}
+
+bool SU_RB_GetBinLength(const char Key[],int *length)
+{
+  fprintf(stderr,"SU_RB_GetBinLength : TO DO !! Abort\n");
+  abort();
 }
 
 bool SU_RB_SetStrValue(const char Key[],const char Value[])
@@ -1156,6 +1394,12 @@ bool SU_RB_SetIntValue(const char Key[],int Value)
   }
 }
 
+bool SU_RB_SetBinValue(const char Key[],const char *Value,int val_len)
+{
+  fprintf(stderr,"SU_RB_SetBinValue : TO DO !! Abort\n");
+  abort();
+}
+
 bool SU_RB_DelKey(const char Key[])
 {
   SU_RB_PNode Handle;
