@@ -1,6 +1,6 @@
 /******************************************************************************/
 /* Skyutils functions - Chained list,socket,string,utils,web,threads, archive */
-/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-04                             */
+/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-05                             */
 /******************************************************************************/
 
 /*
@@ -23,7 +23,7 @@
 #ifndef __SKY_UTILS_H__
 #define __SKY_UTILS_H__
 
-#define SKYUTILS_VERSION "2.7"
+#define SKYUTILS_VERSION "2.8"
 #define SKYUTILS_AUTHOR "Christophe Calmejane"
 
 #if defined(__MACH__) || defined(_AIX)
@@ -107,16 +107,7 @@ typedef signed __int32 SU_s32;
 typedef unsigned __int64 SU_u64;
 typedef signed __int64 SU_s64;
 #else /* !(_MSC_VER || __BORLANDC__) */
-#if defined(_WIN32) && defined(__GNUC__) && defined(__MINGW32__)
-typedef unsigned char SU_u8;
-typedef signed char SU_s8;
-typedef unsigned short SU_u16;
-typedef signed short SU_s16;
-typedef unsigned int SU_u32;
-typedef signed int SU_s32;
-typedef unsigned long long SU_u64;
-typedef signed long long SU_s64;
-#else /* Not cygwin in win32 */
+#if defined(__GNUC__) && defined (__linux__) && !defined(_WIN32) && !defined(__MINGW32__)
 typedef u_int8_t SU_u8;
 typedef int8_t SU_s8;
 typedef u_int16_t SU_u16;
@@ -125,9 +116,19 @@ typedef u_int32_t SU_u32;
 typedef int32_t SU_s32;
 typedef u_int64_t SU_u64;
 typedef int64_t SU_s64;
-#endif /* Cygwin on win32 */
+#else /* Not linux GCC */
+typedef unsigned char SU_u8;
+typedef signed char SU_s8;
+typedef unsigned short SU_u16;
+typedef signed short SU_s16;
+typedef unsigned int SU_u32;
+typedef signed int SU_s32;
+typedef unsigned long long SU_u64;
+typedef signed long long SU_s64;
+#endif /* Linux GCC */
 #endif /* _MSC_VER || __BORLANDC__ */
 
+
 /* **************************************** */
 /*            Chained list functions        */
 /* **************************************** */
@@ -218,6 +219,7 @@ char *SU_nocasestrstr(char *text, char *tofind);  /* like strstr(), but nocase *
 bool SU_strwcmp(const char *s,const char *wild); /* True if wild equals s (wild may use '*') */
 bool SU_nocasestrwcmp(const char *s,const char *wild); /* Same as strwcmp but without case */
 bool SU_strwparse(const char *s,const char *wild,char buf[],int size,char *buf_ptrs[],int *ptrs_count); /* True if wild equals s (wild may use '*') */
+bool SU_nocasestrwparse(const char *s,const char *wild,char buf[],int size,char *buf_ptrs[],int *ptrs_count); /* Same as nocasestrwparse but without case */
 unsigned int SU_strnlen(const char *s,unsigned int max); /* Returns MAX(length of a string,max) (not including terminating null char) */
 bool SU_ReadLine(FILE *fp,char S[],int len); /* Returns false on EOF */
 bool SU_ParseConfig(FILE *fp,char Name[],int nsize,char Value[],int vsize); /* Returns false on EOF */
@@ -279,7 +281,7 @@ void SU_PrintDebug(int Level,char *Txt, ...);
 #include <syslog.h>
 #define SU_SYSLOG_FN(x,y) syslog(x,y)
 #else /* !__unix__ */
-#define SU_PrintDebug() /* */
+#define SU_PrintDebug() /* Avoid compilation warnings */
 extern FILE *SU_LogFile;
 #define SU_SYSLOG_FN(x,y) SU_WriteToLogFile(SU_LogFile,y)
 #endif /* __unix__ */
@@ -321,6 +323,7 @@ typedef unsigned long SU_CPUSPEED;
 SU_CPUSPEED SU_GetCPUSpeed(void);
 void SU_GetTicks(SU_TICKS *tick); /* Returned value in processor ticks (Use SU_ElapsedTime to compute time in msec) */
 unsigned long int SU_ElapsedTime(SU_TICKS t1,SU_TICKS t2,SU_CPUSPEED speed); /* Returned value in msec */
+FILE *SU_fmemopen(void *buf,size_t size,const char *opentype);
 
 
 /* **************************************** */
@@ -415,6 +418,14 @@ typedef struct
   void (*OnErrorSendingFile)(int,void *); /* User's CallBack for an error sending file (errno code passed) */
 } SU_THTTP_CB, *SU_PHTTP_CB;
 
+typedef struct
+{
+  char *Header; /* Header to send */
+  char *FileName; /* File to send (exclude Data) */
+  char *Data; /* Data to send (exclude FileName) */
+  int Length; /* Length of data */
+} SU_THTTPPart, *SU_PHTTPPart;
+
 typedef struct SU_SHTTPActions
 { /* Info to set BEFORE any call to ExecuteActions */
   int  Command; /* ACT_xxx */
@@ -422,8 +433,10 @@ typedef struct SU_SHTTPActions
   char *URL_Params; /* ACT_GET & ACT_POST */
   char *Post_Data;  /* ACT_POST */
   int  Post_Length; /* ACT_POST */
+  char *ContentType; /* Content-type or use default */ /* ACT_GET & ACT_POST */
   char *FileName;   /* ACT_PUT */ /* URL must contain the URL+New file name */ /* If defined for GET or POST, dump result to this file */
   char *Referer;
+  SU_PList MultiParts; /* SU_PHTTPPart */ /* MultiParts, exclude Post_Data */ /* ACT_POST */
   void *User;       /* User's info Passed to Callbacks */
   int  Sleep;       /* Time to wait before sending command (sec) */
   SU_THTTP_CB CB;   /* Callbacks structure */
@@ -464,6 +477,8 @@ typedef struct
   SU_PList Inputs;
 } SU_TForm, *SU_PForm;
 
+/* Sets the UserAgent string to send in every web request. Will not be used if SU_LoadUserHeaderFile() is used */
+void SU_SetUserAgent(const char UA[]);
 /* Sets proxy server,port, user and password values to be used by ExecuteActions (use NULL for proxy to remove use of the proxy) */
 void SU_SetProxy(const char Proxy[],const int Port,const char User[], const char Password[]);
 /* Sets the socket connection timeout (use 0 to reset default value) */
@@ -696,16 +711,22 @@ SU_AR_COMP_TYPE SU_AR_SupportedComps(void);
 #define SU_RB_ERR_WRITE_ERROR   7
 #define SU_RB_ERR_INVALID_TYPE  8
 #define SU_RB_ERR_CORRUPTED     9
+#define SU_RB_ERR_MORE_DATA    10
+#define SU_RB_ERR_NO_SUCH_KEY  11
 
 int SU_RB_GetLastError();
 
 /* 'Key' parameter must be a succession of subkeys separated by '\', and finished by a keyname. To set/get the default value of a subkey, put a trailing '\' on 'Key' */
-void SU_RB_GetStrValue(const char Key[],char *buf,int buf_len,const char Default[]);
-int SU_RB_GetIntValue(const char Key[],int Default);
-bool SU_RB_SetStrValue(const char Key[],const char Value[]);
-bool SU_RB_SetIntValue(const char Key[],int Value);
-bool SU_RB_DelKey(const char Key[]); /* Recursively deletes All sub-keys and values of Key (and Key itself) */
-bool SU_RB_DelValue(const char Key[]);
+bool SU_RB_GetStrValue(const char Key[],char *buf,int buf_len,const char Default[]); /* True on success. Call SU_RB_GetLastError() for error */
+bool SU_RB_GetStrLength(const char Key[],int *length); /* True on success. Call SU_RB_GetLastError() for error */
+bool SU_RB_GetIntValue(const char Key[],int *Value,int Default); /* True on success. Call SU_RB_GetLastError() for error */
+bool SU_RB_GetBinValue(const char Key[],char *buf,int buf_len,char *Default,int def_len); /* True on success. Call SU_RB_GetLastError() for error */
+bool SU_RB_GetBinLength(const char Key[],int *length); /* True on success. Call SU_RB_GetLastError() for error */
+bool SU_RB_SetStrValue(const char Key[],const char Value[]); /* True on success. Call SU_RB_GetLastError() for error */
+bool SU_RB_SetIntValue(const char Key[],int Value); /* True on success. Call SU_RB_GetLastError() for error */
+bool SU_RB_SetBinValue(const char Key[],const char *Value,int val_len); /* True on success. Call SU_RB_GetLastError() for error */
+bool SU_RB_DelKey(const char Key[]); /* Recursively deletes All sub-keys and values of Key (and Key itself) */ /* True on success. Call SU_RB_GetLastError() for error */
+bool SU_RB_DelValue(const char Key[]); /* True on success. Call SU_RB_GetLastError() for error */
 
 bool SU_RB_OpenRegistry(const char RegistryPath[]); /* Opens registry for reading/writing. True on success - Only one registry opened at one time, close previous one if opened */
 bool SU_RB_CloseRegistry(void); /* Flush all writings since registry was opened and close it. True on success. If failed, nothing was flushed */
@@ -718,6 +739,7 @@ bool SU_RB_EnumIntValue(HKEY Key,int Idx,char *Name,int name_len,int *Value); /*
 void SU_RB_CloseKey(HKEY Key);
 #endif /* !SU_INCLUDE_NO_REG */
 
+
 /* **************************************** */
 /*              Debug  functions            */
 /* **************************************** */
@@ -731,23 +753,23 @@ void SU_RB_CloseKey(HKEY Key);
 #define SU_DBG_CONSOLE_DEFAULT_WINDOW_NAME "SkyUtils_Debug_Window"
 
 /* ** Global options */
-void SU_DBG_SetFlags(const SU_u64 Flags);
-void SU_DBG_SetOutput(const SU_u16 Output);
+void SU_DBG_SetFlags(const SU_u64 Flags); /* Effective immediatly */
+void SU_DBG_SetOutput(const SU_u16 Output); /* Effective immediatly */
 /* If PrintTime is set to true, date/time is also printed. CurrentProcessId/CurrentThreadId is printed if PrintProcessId/PrintThreadId is set to true */
-void SU_DBG_SetOptions(const bool PrintTime,const bool PrintProcessId,const bool PrintThreadId);
+void SU_DBG_SetOptions(const bool PrintTime,const bool PrintProcessId,const bool PrintThreadId); /* Effective immediatly */
 
 /* ** Printf Output Options ** */
 /* Ansi color is used if AnsiColor is set to true */
-void SU_DBG_OUT_PRINTF_SetOptions(const bool AnsiColor);
+void SU_DBG_OUT_PRINTF_SetOptions(const bool AnsiColor); /* Effective immediatly */
 /* ** Console Output Options ** */
 /* WindowName is the name of the debug console window. Set it to NULL to stop sending messages to it */
-void SU_DBG_OUT_CONSOLE_SetOptions(const char WindowName[]);
+void SU_DBG_OUT_CONSOLE_SetOptions(const char WindowName[]); /* Effective immediatly (changes the Console immediatly) */
 /* ** File Output Options ** */
 /* FileName is the name of the log file. Set it to NULL to close log file. File is deleted if DeletePreviousLog is set to true */
-void SU_DBG_OUT_FILE_SetOptions(const char FileName[],bool DeletePreviousLog);
+void SU_DBG_OUT_FILE_SetOptions(const char FileName[],bool DeletePreviousLog); /* Effective immediatly (closes old file if any, opens the new one) */
 /* ** Socket Output Options ** */
 /* HostName:Port is the Host and port to connect to. Set Port to 0, to remove this host from list of socket */
-void SU_DBG_OUT_SOCKET_SetOptions(const char HostName[],const int Port);
+void SU_DBG_OUT_SOCKET_SetOptions(const char HostName[],const int Port); /* Effective immediatly (adds a new socket) */
 
 /* PrintDebug fonction. \n is added a the end of the string */
 void SU_DBG_PrintDebug(const SU_u64 Type,char *Txt, ...);
@@ -779,7 +801,8 @@ bool SU_DBG_ChooseDebugOptions_Windows(HINSTANCE hInstance,HWND Parent);
 
 /* Dummy functions used by configure, to check correct version of skyutils */
 /* Remove old ones if compatibility has been broken */
-void SU_Dummy207(void);
+void SU_Dummy208(void);
+
 
 #if defined(__cplusplus) && !defined(__BORLANDC__)
 }
