@@ -1,6 +1,6 @@
 /****************************************************************/
 /* String unit                                                  */
-/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-04               */
+/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-05               */
 /****************************************************************/
 
 /*
@@ -306,6 +306,20 @@ bool SU_strwparse(const char *s,const char *wild,char buf[],int size,char *buf_p
   return false;
 }
 
+bool SU_nocasestrwparse(const char *s,const char *wild,char buf[],int size,char *buf_ptrs[],int *ptrs_count) /* True if wild equals s (wild may use '*') */
+{
+  char *tmp = strdup(s);
+  char *tmp_wild = strdup(wild);
+  bool ret;
+  
+  SU_strtolower(tmp);
+  SU_strtolower(tmp_wild);
+  ret = SU_strwparse(tmp,tmp_wild,buf,size,buf_ptrs,ptrs_count);
+  free(tmp);
+  free(tmp_wild);
+  return ret;
+}
+
 
 /* Returns MAX(length of a string,max) (not including terminating null char) */
 unsigned int SU_strnlen(const char *s,unsigned int max)
@@ -494,7 +508,7 @@ unsigned char SU_toupper(unsigned char c)
 {
   if((c >= 'a') && (c <= 'z'))
     return (c-32);
-  if((c >= 224) && (c <= 255))
+  if(c >= 224)
     return (c-32);
   return c;
 }
