@@ -1,6 +1,6 @@
 /****************************************************************/
 /* Utils unit                                                   */
-/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-04               */
+/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-05               */
 /****************************************************************/
 
 /*
@@ -473,7 +473,7 @@ void *SU_DL_GetSym(SU_DL_HANDLE handle, const char Name[])
 /* PRECISE COUNTER FUNCTIONS */
 #if defined(_MSC_VER) || defined(__BORLANDC__)
 #ifndef _M_IX86
-#error Unsupported architecture : Win32 && !ix86 (contact zekiller@skytech.org)
+#error Unsupported architecture : Win32 && !ix86 (contact zekiller[AT]skytech[DOT]org)
 #endif /* !_M_IX86 */
 
 SU_CPUSPEED SU_GetCPUSpeed(void)
@@ -632,4 +632,21 @@ unsigned long int SU_ElapsedTime(SU_TICKS t1,SU_TICKS t2,SU_CPUSPEED speed)
 #endif /* _WIN32 */
 
 
-void SU_Dummy207(void) {}
+FILE *SU_fmemopen(void *buf,size_t size,const char *opentype)
+{
+  FILE *f;
+
+  if(opentype[0] != 'r')
+    return NULL;
+
+  f = tmpfile();
+  fwrite(buf,1,size,f);
+  rewind(f);
+
+  return f;
+}
+
+
+void SU_Dummy208(void) {}
+
+
