@@ -1,6 +1,6 @@
 /****************************************************************/
 /* Web unit                                                     */
-/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-04               */
+/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-05               */
 /****************************************************************/
 
 /*
@@ -34,8 +34,8 @@
 #undef free
 #endif /* !SU_TRACE_INTERNAL */
 
-#define USER_AGENT "Mozilla/6.0 (compatible; MSIE 5.01; Windows NT)"
-#define SW_DEFAULT_HEADER "User-Agent: " USER_AGENT "\x0D" "\x0A" "Connection: Keep-Alive" "\x0D" "\x0A" "Accept-Language: fr-FR, en" "\x0D" "\x0A" "Accept-Charset: iso-8859-1,*,utf-8" "\x0D" "\x0A" "Accept: text/html, text/plain, text/*, image/gif, image/jpg, image/png, */*" "\x0D" "\x0A"
+#define SW_DEFAULT_USER_AGENT "Mozilla/6.0 (compatible; MSIE 5.01; Windows NT)"
+#define SW_DEFAULT_HEADER "Connection: Keep-Alive" "\x0D" "\x0A" "Accept-Language: fr-FR, en" "\x0D" "\x0A" "Accept-Charset: iso-8859-1,*,utf-8" "\x0D" "\x0A" "Accept: text/html, text/plain, text/*, image/gif, image/jpg, image/png, */*" "\x0D" "\x0A"
 #define DEFAULT_PORT 80
 #define DEFAULT_SSL_PORT 443
 #define SOCKET_TIME_OUT 60
@@ -50,6 +50,7 @@ char *SW_Proxy_String = NULL;
 char *SW_Proxy_User = NULL;
 char *SW_Proxy_Password = NULL;
 char *SW_UserHeader = NULL;
+char *SW_UserAgent = NULL;
 int   SW_Proxy_Port = 0;
 int   SW_SocketTimeout = SOCKET_TIME_OUT;
 SU_PList SW_Cookies = NULL; /* SU_PCookie */
@@ -619,12 +620,14 @@ SU_PAnswer ParseBuffer(SU_PAnswer Ans,char *Buf,int *len,SU_PHTTPActions Act,boo
     Ans = (SU_PAnswer) malloc(sizeof(SU_TAnswer));
     memset(Ans,0,sizeof(SU_TAnswer));
     Ans->Data_Length = -1;
+    Ans->Data_ToReceive = -1;
   }
   if(Ans->Data_Length != -1)
   {
-    Ans->Data = (char *) realloc(Ans->Data,Ans->Data_Length+*len);
+    Ans->Data = (char *) realloc(Ans->Data,Ans->Data_Length+*len+1); /* +1 for \0 */
     memcpy(Ans->Data+Ans->Data_Length,Buf,*len);
     Ans->Data_Length += *len;
+    Ans->Data[Ans->Data_Length] = 0;
     *len = 0;
     return Ans;
   }
@@ -639,16 +642,17 @@ SU_PAnswer ParseBuffer(SU_PAnswer Ans,char *Buf,int *len,SU_PHTTPActions Act,boo
       if(SU_DebugLevel >= 3)
       {
         printf("SkyUtils_ParseBuffer : Found Data in HTTP answer\n");
-        if(Ans->Data_ToReceive != 0)
+        if(Ans->Data_ToReceive >= 0)
           printf("SkyUtils_ParseBuffer : Waiting %d bytes\n",Ans->Data_ToReceive);
       }
 #endif /* __unix__ */
       Ans->Data_Length = 0;
       if(*len == 2) /* Not enough data */
         return Ans;
-      Ans->Data = (char *) malloc(*len-2);
+      Ans->Data = (char *) malloc(*len-2+1); /* +1 for \0 */
       memcpy(Ans->Data,Buf+2,*len-2);
       Ans->Data_Length = *len - 2;
+      Ans->Data[Ans->Data_Length] = 0;
       *len = 0;
       return Ans;
     }
@@ -880,10 +884,108 @@ char * http_base64_encode(const char *text)
   return buffer;
 }
 
+static int SendBuffer(int Sock,char *buf,int len,SSL *ssl,bool verbose)
+{
+  int res;
+
+#ifdef SU_USE_SSL
+  if(ssl)
+  {
+    if(verbose)
+    {
+      char errormsg[1024];
+      res = SU_SSL_Write(ssl,buf,len,errormsg);
+      if(res == -1)
+      {
+        printf("SkyUtils_SendCommand Error : Error sending command using SSL : %s\n",errormsg);
+      }
+    }
+    else
+      res = SU_SSL_Write(ssl,buf,len,NULL);
+  }
+  else
+#endif /* SU_USE_SSL */
+#ifdef __unix__
+  if(SU_DebugLevel >= 2)
+    printf("SkyUtils_SendCommand : Sending %s(%d) : %s\n",ssl?"SSL ":"",len,buf);
+#endif /* __unix__ */
+  res = send(Sock,buf,len,0);
+  return res;
+}
+
+static int SendFile(int Sock,FILE *fp,int FLen,SSL *ssl)
+{
+  int res = 0;
+  char buf[16000];
+  int len,pos;
+
+  while(res >= 0)
+  {
+    len = (FLen > sizeof(buf))?sizeof(buf):FLen;
+    if(fread(buf,len,1,fp) != 1)
+    {
+      res = -1;
+      break;
+    }
+#ifdef SU_USE_SSL
+    if(ssl)
+      res = SU_SSL_Write(ssl,buf,len,NULL);
+    else
+#endif /* SU_USE_SSL */
+    res = send(Sock,buf,len,SU_MSG_NOSIGNAL);
+    FLen -= len;
+    if(res <= 0)
+    {
+      res = -1;
+      break;
+    }
+    else if(res != len)
+    { /* Not all bytes sent */
+      pos = res;
+      len -= res;
+      while(len > 0)
+      {
+#ifdef SU_USE_SSL
+        if(ssl)
+          res = SU_SSL_Write(ssl,buf+pos,len,NULL);
+        else
+#endif /* SU_USE_SSL */
+        res = send(Sock,buf+pos,len,SU_MSG_NOSIGNAL);
+        if(res <= 0)
+          break;
+        pos += res;
+        len -= res;
+      }
+      if(res <= 0)
+        break;
+    }
+    if(FLen == 0)
+    {
+      len = 0;
+      buf[len++] = 0x0D;
+      buf[len++] = 0x0A;
+      buf[len] = 0;
+#ifdef SU_USE_SSL
+      if(ssl)
+        res = SU_SSL_Write(ssl,buf,len,NULL);
+      else
+#endif /* SU_USE_SSL */
+      send(Sock,buf,len,SU_MSG_NOSIGNAL);
+#ifdef __unix__
+      if(SU_DebugLevel >= 2)
+        printf("SkyUtils_SendCommand : Successfully sent file\n");
+#endif /* __unix__ */
+      res = 0;
+      break;
+    }
+  }
+  return res;
+}
+
 bool SendCommand(SU_SOCKET Sock,SU_PHTTPActions Act,bool proxy,SSL *ssl)
 {
   char buf[16000];
-  int len,pos;
+  int len;
   long int FLen;
   int res;
   char *Com,*tmp,*tmp2,*tmp3;
@@ -909,7 +1011,11 @@ bool SendCommand(SU_SOCKET Sock,SU_PHTTPActions Act,bool proxy,SSL *ssl)
   len = strlen(buf);
   /* Now add header from file, or default one */
   if(SW_UserHeader == NULL)
-    snprintf(buf+len,sizeof(buf)-len,"%s",SW_DEFAULT_HEADER);
+  {
+    if(SW_UserAgent == NULL)
+      SU_SetUserAgent(SW_DEFAULT_USER_AGENT);
+    snprintf(buf+len,sizeof(buf)-len,"User-Agent: %s\x0D\x0A%s",SW_UserAgent,SW_DEFAULT_HEADER);
+  }
   else
     snprintf(buf+len,sizeof(buf)-len,"%s",SW_UserHeader);
   len = strlen(buf);
@@ -996,19 +1102,117 @@ bool SendCommand(SU_SOCKET Sock,SU_PHTTPActions Act,bool proxy,SSL *ssl)
   }
   if(Act->Command == ACT_POST)
   {
-    snprintf(buf+len,sizeof(buf),"Content-type: application/x-www-form-urlencoded%c%cContent-length: %d%c%c%c%c",0x0D,0x0A,Act->Post_Length,0x0D,0x0A,0x0D,0x0A);
-    len = strlen(buf);
-    memcpy(buf+len,Act->Post_Data,Act->Post_Length);
-    len += Act->Post_Length;
-    buf[len++] = 0x0D;
-    buf[len++] = 0x0A;
-    buf[len] = 0;
+    if(Act->MultiParts == NULL) /* Post_Data */
+    {
+      snprintf(buf+len,sizeof(buf),"Content-type: application/x-www-form-urlencoded%c%cContent-length: %d%c%c%c%c",0x0D,0x0A,Act->Post_Length,0x0D,0x0A,0x0D,0x0A);
+      len = strlen(buf);
+      memcpy(buf+len,Act->Post_Data,Act->Post_Length);
+      len += Act->Post_Length;
+      buf[len++] = 0x0D;
+      buf[len++] = 0x0A;
+      buf[len] = 0;
+      SendBuffer(Sock,buf,len,ssl,true);
+    }
+    else /* MultiParts */
+    {
+      int multi_length = 0;
+      SU_PList Ptr = Act->MultiParts;
+      SU_PHTTPPart Part;
+      char boundary[27+13+1];
+      int boundary_length;
+      unsigned long int tim = time(NULL);
+      unsigned long int pid = SU_PROCESS_SELF;
+      unsigned long int tid = SU_THREAD_SELF;
+      if(tim >= 0x1000000)
+        tim &= 0xFFFFFF;
+      if(pid >= 0x10000)
+        pid &= 0xFFFF;
+      if(tid >= 0x1000)
+        tid &= 0xFFF;
+
+      snprintf(boundary,sizeof(boundary),"---------------------------%06x%04x%03x",tim,pid,tid);
+      boundary_length = strlen(boundary);
+      while(Ptr != NULL)
+      {
+        Part = (SU_PHTTPPart) Ptr->Data;
+        if(Part->FileName)
+        {
+          fp = fopen(Part->FileName,"rb");
+          if(fp == NULL)
+            return false;
+          fseek(fp,0,SEEK_END);
+          Part->Length = ftell(fp);
+          fclose(fp);
+        }
+        multi_length += Part->Length + boundary_length + 2 + 2 + 2; /* +2 (--) before boundary, +2 (\n) after boundary, +2 (\n) at end of part */
+        if(Part->Header)
+        {
+          multi_length += strlen(Part->Header) + 2; /* +2 after header */
+        }
+        multi_length += 2; /* +2 (--) for final boundary */
+        Ptr = Ptr->Next;
+      }
+      snprintf(buf+len,sizeof(buf),"Content-type: multipart/form-data; boundary=%s%c%cContent-length: %d%c%c%c%c",boundary,0x0D,0x0A,multi_length,0x0D,0x0A,0x0D,0x0A);
+      len = strlen(buf);
+      buf[len] = 0;
+      SendBuffer(Sock,buf,len,ssl,true);
+
+      Ptr = Act->MultiParts;
+      while(Ptr != NULL)
+      {
+        Part = (SU_PHTTPPart) Ptr->Data;
+        if(Part->Header)
+        {
+          snprintf(buf,sizeof(buf),"--%s%c%c%s%c%c%c%c",boundary,0x0D,0x0A,Part->Header,0x0D,0x0A,0x0D,0x0A);
+          len = strlen(buf);
+          buf[len] = 0;
+        }
+        else
+        {
+          snprintf(buf,sizeof(buf),"--%s%c%c%c%c",boundary,0x0D,0x0A,0x0D,0x0A);
+          len = strlen(buf);
+          buf[len] = 0;
+        }
+        SendBuffer(Sock,buf,len,ssl,true);
+        if(Part->FileName) /* File */
+        {
+          fp = fopen(Part->FileName,"rb");
+          if(fp == NULL)
+            return false;
+          res = SendFile(Sock,fp,Part->Length,ssl);
+          fclose(fp);
+          if(res == -1)
+            return false;
+        }
+        else /* Data */
+        {
+          SendBuffer(Sock,Part->Data,Part->Length,ssl,false);
+        }
+        /* End of boundary */
+        snprintf(buf,sizeof(buf),"%c%c",0x0D,0x0A);
+        len = strlen(buf);
+        buf[len] = 0;
+        SendBuffer(Sock,buf,len,ssl,true);
+        Ptr = Ptr->Next;
+      }
+      /* Final boundary */
+      snprintf(buf,sizeof(buf),"--%s--%c%c",boundary,0x0D,0x0A);
+      len = strlen(buf);
+      buf[len] = 0;
+      SendBuffer(Sock,buf,len,ssl,true);
+    }
   }
   else if((Act->Command == ACT_GET) || (Act->Command == ACT_DELETE))
   {
+    if(Act->ContentType != NULL)
+    {
+      snprintf(buf+len,sizeof(buf),"Content-Type: %s%c%c",Act->ContentType,0x0D,0x0A);
+      len = strlen(buf);
+    }
     buf[len++] = 0x0D;
     buf[len++] = 0x0A;
     buf[len] = 0;
+    SendBuffer(Sock,buf,len,ssl,true);
   }
   if(Act->Command == ACT_PUT)
   {
@@ -1022,68 +1226,11 @@ bool SendCommand(SU_SOCKET Sock,SU_PHTTPActions Act,bool proxy,SSL *ssl)
     if(SU_DebugLevel >= 2)
       printf("SkyUtils_SendCommand : Sending file %s of length %ld\n",Act->FileName,FLen);
 #endif /* __unix__ */
-    snprintf(buf+len,sizeof(buf),"Content-length: %ld%c%c%c%c",FLen,0x0D,0x0A,0x0D,0x0A);
+    snprintf(buf+len,sizeof(buf),"Content-Type: application/octet-stream%c%cContent-length: %ld%c%c%c%c",FLen,0x0D,0x0A,0x0D,0x0A,0x0D,0x0A);
     len = strlen(buf);
-#ifdef SU_USE_SSL
-    if(ssl)
-      res = SU_SSL_Write(ssl,buf,len,NULL);
-    else
-#endif /* SU_USE_SSL */
-    res = send(Sock,buf,len,SU_MSG_NOSIGNAL);
-    while(res >= 0)
-    {
-      len = (FLen > sizeof(buf))?sizeof(buf):FLen;
-      if(fread(buf,len,1,fp) != 1)
-        break;
-#ifdef SU_USE_SSL
-      if(ssl)
-        res = SU_SSL_Write(ssl,buf,len,NULL);
-      else
-#endif /* SU_USE_SSL */
-      res = send(Sock,buf,len,SU_MSG_NOSIGNAL);
-      FLen -= len;
-      if(res <= 0)
-        break;
-      else if(res != len)
-      { /* Not all bytes sent */
-        pos = res;
-        len -= res;
-        while(len > 0)
-        {
-#ifdef SU_USE_SSL
-          if(ssl)
-            res = SU_SSL_Write(ssl,buf+pos,len,NULL);
-          else
-#endif /* SU_USE_SSL */
-          res = send(Sock,buf+pos,len,SU_MSG_NOSIGNAL);
-          if(res <= 0)
-            break;
-          pos += res;
-          len -= res;
-        }
-        if(res <= 0)
-          break;
-      }
-      if(FLen == 0)
-      {
-        fclose(fp);
-        len = 0;
-        buf[len++] = 0x0D;
-        buf[len++] = 0x0A;
-        buf[len] = 0;
-#ifdef SU_USE_SSL
-        if(ssl)
-          res = SU_SSL_Write(ssl,buf,len,NULL);
-        else
-#endif /* SU_USE_SSL */
-        send(Sock,buf,len,SU_MSG_NOSIGNAL);
-#ifdef __unix__
-        if(SU_DebugLevel >= 2)
-          printf("SkyUtils_SendCommand : Successfully sent file\n");
-#endif /* __unix__ */
-        return true;
-      }
-    }
+    res = SendBuffer(Sock,buf,len,ssl,true);
+    res = SendFile(Sock,fp,FLen,ssl);
+    fclose(fp);
     if(res == -1)
     {
       if(Act->CB.OnErrorSendingFile != NULL)
@@ -1093,7 +1240,6 @@ bool SendCommand(SU_SOCKET Sock,SU_PHTTPActions Act,bool proxy,SSL *ssl)
         printf("SkyUtils_SendCommand Warning : Error sending file, %ld bytes remaining not sent\n",FLen);
 #endif /* __unix__ */
     }
-    fclose(fp);
 #ifdef SU_USE_SSL
     if(ssl)
     {
@@ -1101,25 +1247,8 @@ bool SendCommand(SU_SOCKET Sock,SU_PHTTPActions Act,bool proxy,SSL *ssl)
     }
 #endif /* SU_USE_SSL */
     SU_CLOSE_SOCKET(Sock);
-    return false;
+    return res == 0;
   }
-#ifdef __unix__
-  if(SU_DebugLevel >= 2)
-    printf("SkyUtils_SendCommand : Sending %s(%d) : %s\n",ssl?"SSL ":"",len,buf);
-#endif /* __unix__ */
-#ifdef SU_USE_SSL
-  if(ssl)
-  {
-    char errormsg[1024];
-    res = SU_SSL_Write(ssl,buf,len,errormsg);
-    if(res == -1)
-    {
-      printf("SkyUtils_SendCommand Error : Error sending command using SSL : %s\n",errormsg);
-    }
-  }
-  else
-#endif /* SU_USE_SSL */
-  res = send(Sock,buf,len,0);
   return true;
 }
 
@@ -1151,7 +1280,7 @@ SU_PAnswer WaitForAnswer(SU_SOCKET Sock,SU_PHTTPActions Act,bool proxy,SSL *ssl)
     len += BufPos;
     Ans = ParseBuffer(Ans,Buf,&len,Act,proxy);
     BufPos = len;
-    if(Ans->Data_ToReceive != 0)
+    if(Ans->Data_ToReceive >= 0)
     {
       if(Ans->Data_Length >= Ans->Data_ToReceive)
         break;
@@ -1224,7 +1353,9 @@ bool SU_SendProxySSLConnect(SU_SOCKET Sock,char *Host,int Port,int *Code)
   char *ptr;
   float f;
 
-  snprintf(buf,sizeof(buf),"CONNECT %s:%d HTTP/1.0%c%cUser-Agent: %s%c%cHost: %s%c%cProxy-Connection: close%c%cConnection: close%c%c%c%c",Host,Port,0x0D,0x0A,USER_AGENT,0x0D,0x0A,Host,0x0D,0x0A,0x0D,0x0A,0x0D,0x0A,0x0D,0x0A);
+  if(SW_UserAgent == NULL)
+    SU_SetUserAgent(SW_DEFAULT_USER_AGENT);
+  snprintf(buf,sizeof(buf),"CONNECT %s:%d HTTP/1.0%c%cUser-Agent: %s%c%cHost: %s%c%cProxy-Connection: close%c%cConnection: close%c%c%c%c",Host,Port,0x0D,0x0A,SW_UserAgent,0x0D,0x0A,Host,0x0D,0x0A,0x0D,0x0A,0x0D,0x0A,0x0D,0x0A);
   len = strlen(buf);
   res = send(Sock,buf,len,0);
   if(res != len)
@@ -1281,10 +1412,10 @@ bool SU_SendProxySSLConnect(SU_SOCKET Sock,char *Host,int Port,int *Code)
 char *SU_EncodeURL(const char URL_in[],char URL_out[],int URL_out_len)
 {
   char NB[10];
-  int i,pos,val;
+  int i,pos;
 
   pos = 0;
-  for(i=0;i<strlen(URL_in);i++)
+  for(i=0;i<(int)strlen(URL_in);i++)
   {
     if((URL_in[i] >='!') && (URL_in[i] <='~'))
     {
@@ -1504,6 +1635,17 @@ int SU_ExecuteActions(SU_PList Actions)
   return 0;
 }
 
+void SU_FreePart(SU_PHTTPPart Part)
+{
+  if(Part->Header)
+    free(Part->Header);
+  if(Part->FileName)
+    free(Part->FileName);
+  if(Part->Data)
+    free(Part->Data);
+  free(Part);
+}
+
 void SU_FreeAction(SU_PHTTPActions Act)
 {
   if(Act->URL_Params != NULL)
@@ -1514,6 +1656,18 @@ void SU_FreeAction(SU_PHTTPActions Act)
     free(Act->FileName);
   if(Act->Referer != NULL)
     free(Act->Referer);
+  if(Act->ContentType)
+    free(Act->ContentType);
+  if(Act->MultiParts)
+  {
+    SU_PList Ptr = Act->MultiParts;
+    while(Ptr != NULL)
+    {
+      SU_FreePart(Ptr->Data);
+      Ptr = Ptr->Next;
+    }
+    SU_FreeList(Act->MultiParts);
+  }
   free(Act);
 }
 
@@ -2356,3 +2510,12 @@ char *SU_GetStringFromHtml(const char Ans[],const char TextBefore[])
   return tmp;
 }
 
+void SU_SetUserAgent(const char UA[])
+{
+  if(SW_UserAgent != NULL)
+    free(SW_UserAgent);
+  if(UA == NULL)
+    SW_UserAgent = NULL;
+  else
+    SW_UserAgent = strdup(UA);
+}
